/* a simple demo for changing the bootloader

   1. start the bootloader by HW contition 
   2. download changer_iap.hex to the device 
   3. enable IAP whith v1_tools.exe
   
   It will work on V1.1 only. The program first checks boootloader bit and 
   flashes the image to 0x3800. The image was created with all options 
   exept REMOVE_SERIAL_CODE switched on. This bootloader reports 1.11 in the
   BCD_VER field of the devicedescriptor. 
   
   Status LEDS:
   LED1 blinking   : bBOOT_LOAD not set therefore no flashing possible
   LED3 blinking   : nothing to flash Loader == Image
   LED1 + LED3     : same blinking     -> verify ok
                   : diferent blinking -> verify fails
   LED1 + LDE3     : on -> Loder startet by SW1                  
*/

#include <absacc.h>
#include <intrins.h>
#include "..\inc\ch552_keil.h"


typedef unsigned short uint16_t; //org Keil does not have these
typedef unsigned char  uint8_t;

  // from Aarons board
  #define LED1   TXD      
  #define LED3   RXD 
  #define SW1    INT0

  #define CALL(addr)   (((void(*)(void))(uint8_t code *)addr)()) //call any address

  
/** a simple delay loop used for LED blinking */  
void delay(uint16_t Time)
{
  while(Time)
  {
     _nop_();
     _nop_();
     _nop_();
     _nop_();  
     Time--;    
  }    
}  

/** store one word into the codeflash */
uint8_t WriteCodeFlash(uint16_t Address, uint16_t Data)
{
   ROM_ADDR = Address;
   ROM_DATA = Data;
   if (ROM_CTRL & bROM_ADDR_OK)
   {
      E_DIS = 1;
      ROM_CTRL = ROM_CMD_WRITE;
      E_DIS = 0;
      return ROM_CTRL ^ bROM_ADDR_OK;
   }           
   return bROM_ADDR_OK;
}

uint16_t crc;

void crc16(uint8_t b)
{
   register uint8_t i;
   crc^=b;
   for(i=8;i;i--)
   {
     crc= (crc>>1)^((crc&1) ? 0xA001:0);
   }
}  

// this must match the declaration in image.c
// a better way would be to create a h file too
// hex2c.exe is very simple
extern code unsigned char Image[0x07F0];

uint16_t BuildCrc(uint8_t *p)
{
   uint16_t i;
   crc=0; 
   for (i=0; i < 0x7F0;i++) // ignore the special memory 
   {
      crc16(*p);
      p++;
   }
   return crc;
}
void main (void)
{
   uint16_t  image_crc;
   uint16_t  loader_crc;
   uint16_t  addr; 
   uint16_t  value;
   uint16_t  org;
   uint16_t  i;   

   if (SW1 == 0) CALL (0x3800); 
   
   LED1 = 0;
   LED3 = 0; 

  if (!(GLOBAL_CFG & bBOOT_LOAD))
   {
      while(1)
      {
         delay(0xFFFF);
         LED1 = 1; 
         delay(0xFFFF);
         LED1 = 0;
      }
   }
   
   crc=0; image_crc =BuildCrc(Image);
   crc=0; loader_crc=BuildCrc((uint8_t code *) 0x3800);
   if(image_crc!=loader_crc)
   {  
      // enable Flash Mode
      SAFE_MOD    = 0x55;
	    SAFE_MOD    = 0xAA;
	    GLOBAL_CFG |= bCODE_WE;
      SAFE_MOD    = 0;
      
      // flash the bootloader 
      addr = 0x3800;
      for (i=0; i<0x7F0;i+=2)
      {          
         value=(Image[i+1] <<8) | Image[i];  
         org = ((CBYTE[0x3800+i+1]) << 8) | CBYTE[0x3800+i]; 
         if (value != org)
           WriteCodeFlash(addr,value);
         addr+=2;
      }
      SAFE_MOD    =  0x55;
	    SAFE_MOD    =  0xAA;
	    GLOBAL_CFG &= ~bCODE_WE;
      
      crc=0; image_crc =BuildCrc(Image);
      crc=0; loader_crc=BuildCrc((uint8_t code *) 0x3800);
      if(image_crc==loader_crc)
      {
         // verify ok 
         while(1) 
         {
            delay(0xFFFF);
            LED1=1;LED3 = 1; 
            delay(0xFFFF);
            LED1=0;LED3 = 0;
         }  
      }
      else 
      {
         // verfiy failed
         while(1)
         {
            delay(0xFFFF);
            LED1=0;LED3 = 1; 
            delay(0xFFFF);
            LED1=1;LED3 = 0;
         }  
      }
   }
   
   // nothing to do
   while(1)
   {
      delay(0xFFFF);
      LED3 = 1; 
      delay(0xFFFF);
      LED3 = 0;
   }
}